/*
   This synchronization pattern is just a slight variation
   on the rendezvous pattern.

   The RootProcess works alone, and then it signals all the
   other processes to work together (in parallel). After the
   other processes rendezvous, the RootProcess works alone
   again, and then signals the others to work in parallel, etc.etc.

   As an exercise, you should replace the processes with threads
   and then implement this synchronization pattern, first using
   semaphores, and then using condition variables.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mpi.h"
#include <windows.h>  // used for Sleep() only

int main(int argc, char** argv)
{
   int my_rank;  /* Rank of process */
   int p;        /* Number of processes */

   int namelen;
   char name[MPI_MAX_PROCESSOR_NAME];
   //int tag = 1;       /* Tag for messages */
   //MPI_Status status; /* Return status for receive */

   int RootProcess = 0;
   int count = 0;
   int workTime;

   MPI_Init(&argc, &argv);
   MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
   MPI_Comm_size(MPI_COMM_WORLD, &p);
   MPI_Get_processor_name(name, &namelen);

   srand(my_rank + time(NULL));  /* seed the random number generator */

   if (my_rank == 0)
   {
      while(count < 20)
      {
         /* Do some "work" */
         count++;
         workTime = 1+ rand()%10;
         printf("%d: Doing %d seconds of work by rank %d of %d on %s...", count, workTime, my_rank, p, name);
         Sleep(workTime*1000);  // Win32
         //sleep(workTime);     // Posix
         printf("...Done\n");

         /* Send a message to every other process to begin working */
         MPI_Bcast(&count, 1, MPI_INT, RootProcess, MPI_COMM_WORLD);

         /* Rendezvous with all the other processes */
         MPI_Barrier(MPI_COMM_WORLD);
      }
   }
   else /* my_rank != 0 */
   {
      while(count < 20)
      {
         /* Wait for a message from the RooProcess */
         MPI_Bcast(&count, 1, MPI_INT, RootProcess, MPI_COMM_WORLD);

         /* Do some "work" */
         workTime = 1+ rand()%10;
         printf("%d: Doing %d seconds of work by rank %d of %d on %s...", count, workTime, my_rank, p, name);
         Sleep(workTime*1000);  // Win32
         //sleep(workTime);     // Posix
         printf("...Done\n");

         /* Rendezvous with all the other processes */
         MPI_Barrier(MPI_COMM_WORLD);
      }
   }

   MPI_Finalize();
   return 0;
}