/*
   This program tests how MPI_Reduce() blocks. Does it block until
   the buffer is reusable (like MPI_Send()) or does it block until
   all of the processes take part in the reduce (like MPI_Barrier())?
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mpi.h"
#include <windows.h>  // used for Sleep() only

int main(int argc, char** argv)
{
   int rank;  /* Rank of process */
   int p;     /* Number of processes */

   int namelen;
   char name[MPI_MAX_PROCESSOR_NAME];

   int maxValue, workTime, i;
   double startTime, endTime, elapsedTime;

   MPI_Init(&argc, &argv);
   MPI_Comm_rank(MPI_COMM_WORLD, &rank);
   MPI_Comm_size(MPI_COMM_WORLD, &p);
   MPI_Get_processor_name(name, &namelen);

   /* identify the console window */
   printf("rank %d of %d on %s\n", rank, p, name);

   startTime = MPI_Wtime();

   /* do some "work" */
   srand((rank+1)*100); /* seed the random number generator */
   workTime = 1+ rand()%15;
   printf("Doing %d seconds of work.\n", workTime);
   for (i = workTime-1; i >=0; i--)
   {
      Sleep(1000);             // Win32
      //sleep(workTime/1000);  // Posix
      printf("      %d\n", i);
   }

   MPI_Reduce(&workTime, &maxValue, 1, MPI_INT, MPI_MAX, 0, MPI_COMM_WORLD);
   //MPI_Allreduce(&workTime, &maxValue, 1, MPI_INT, MPI_MAX, MPI_COMM_WORLD);
   endTime = MPI_Wtime();
   elapsedTime = endTime - startTime;

   printf("maxValue = %d after %g seconds on rank %d of %d\n", maxValue, elapsedTime, rank, p);

   MPI_Finalize();
   return 0;
}
